@@ -105,13 +105,13 @@ void init_bfu(void)
 	if (!F) return;
 	bfu_bg_color = dip_get_color_sRGB(G_BFU_BG_COLOR);
 	bfu_fg_color = dip_get_color_sRGB(G_BFU_FG_COLOR);
-	bfu_style_wb = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, cast_uchar G_BFU_DEFAULT_FONT, 0);
-	bfu_style_wb_b = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, cast_uchar G_BFU_DEFAULT_FONT, 0);
-	bfu_style_bw = g_get_style(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE, cast_uchar G_BFU_DEFAULT_FONT, 0);
-	bfu_style_bw_u = g_get_style(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE, cast_uchar G_BFU_DEFAULT_FONT, FF_UNDERLINE);
-	bfu_style_bw_mono = g_get_style(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE, cast_uchar "monospaced", 0);
-	bfu_style_wb_mono = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, cast_uchar "monospaced", 0);
-	bfu_style_wb_mono_u = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, cast_uchar "monospaced", FF_UNDERLINE);
+	bfu_style_wb = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, 0);
+	bfu_style_wb_b = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, 0);
+	bfu_style_bw = g_get_style(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE, 0);
+	bfu_style_bw_u = g_get_style(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE, FF_UNDERLINE);
+	bfu_style_bw_mono = g_get_style(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE, FF_MONOSPACED);
+	bfu_style_wb_mono = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, FF_MONOSPACED);
+	bfu_style_wb_mono_u = g_get_style(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE, FF_UNDERLINE | FF_MONOSPACED);
 }
 
 #define G_DIALOG_FIELD_WIDTH g_char_width(bfu_style_wb_mono, ' ')
@@ -249,7 +249,19 @@ static void count_menu_size(struct terminal *term, struct menu *menu)
 	int mx = gf_val(4, 2 * (G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER));
 	int my;
 	for (my = 0; my < menu->ni; my++) {
-		int s = txtlen(term, get_text_translation(menu->items[my].text, term)) + txtlen(term, get_text_translation(get_rtext(menu->items[my].rtext), term)) + gf_val(MENU_HOTKEY_SPACE, G_MENU_HOTKEY_SPACE) * (get_text_translation(get_rtext(menu->items[my].rtext), term)[0] != 0) + gf_val(4, 2 * (G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER));
+		int s;
+#ifdef G
+		if (menu->items[my].free_i & MENU_FONT_LIST)  {
+			struct style *st = g_get_style_font(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE,
+				(menu->items[my].free_i & MENU_FONT_LIST_BOLD ? FF_BOLD : 0) |
+				(menu->items[my].free_i & MENU_FONT_LIST_MONO ? FF_MONOSPACED : 0),
+				menu->items[my].data);
+			s = g_text_width(st, menu->items[my].text);
+			g_free_style(st);
+		} else
+#endif
+			s = txtlen(term, get_text_translation(menu->items[my].text, term)) + txtlen(term, get_text_translation(get_rtext(menu->items[my].rtext), term)) + gf_val(MENU_HOTKEY_SPACE, G_MENU_HOTKEY_SPACE) * (get_text_translation(get_rtext(menu->items[my].rtext), term)[0] != 0);
+		s += gf_val(4, 2 * (G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER));
 		if (s > mx) mx = s;
 	}
 	my = gf_val(my, my * G_BFU_FONT_SIZE);
@@ -398,15 +410,40 @@ static void display_menu_item_gfx(struct terminal *term, struct menu *menu, int
 		}
 		restrict_clip_area(dev, &r, menu->x + G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER, y, menu->x + menu->xw - G_MENU_LEFT_BORDER - G_MENU_LEFT_INNER_BORDER, y + G_BFU_FONT_SIZE);
 		if (it == menu->selected) {
+			struct style *style_wb;
+			if (menu->items[it].free_i & MENU_FONT_LIST) {
+				style_wb = g_get_style_font(G_BFU_BG_COLOR, G_BFU_FG_COLOR, G_BFU_FONT_SIZE,
+					(menu->items[it].free_i & MENU_FONT_LIST_BOLD ? FF_BOLD : 0) |
+					(menu->items[it].free_i & MENU_FONT_LIST_MONO ? FF_MONOSPACED : 0),
+					menu->items[it].data);
+			} else {
+				style_wb = bfu_style_wb;
+			}
 			p = menu->x + G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER;
-			g_print_text(dev, p, y, bfu_style_wb, menu->hktxt1[it], &p);
-			g_print_text(dev, p, y, bfu_style_wb, menu->hktxt2[it], &p);
-			g_print_text(dev, p, y, bfu_style_wb, menu->hktxt3[it], &p);
+			g_print_text(dev, p, y, style_wb, menu->hktxt1[it], &p);
+			g_print_text(dev, p, y, style_wb, menu->hktxt2[it], &p);
+			g_print_text(dev, p, y, style_wb, menu->hktxt3[it], &p);
+			if (menu->items[it].free_i & MENU_FONT_LIST) {
+				g_free_style(style_wb);
+			}
 		} else {
+			struct style *style_bw, *style_bw_u;
+			if (menu->items[it].free_i & MENU_FONT_LIST) {
+				style_bw = style_bw_u = g_get_style_font(G_BFU_FG_COLOR, G_BFU_BG_COLOR, G_BFU_FONT_SIZE,
+					(menu->items[it].free_i & MENU_FONT_LIST_BOLD ? FF_BOLD : 0) |
+					(menu->items[it].free_i & MENU_FONT_LIST_MONO ? FF_MONOSPACED : 0),
+					menu->items[it].data);
+			} else {
+				style_bw = bfu_style_bw;
+				style_bw_u = bfu_style_bw_u;
+			}
 			p = menu->x + G_MENU_LEFT_BORDER + G_MENU_LEFT_INNER_BORDER;
-			g_print_text(dev, p, y, bfu_style_bw, menu->hktxt1[it], &p);
-			g_print_text(dev, p, y, bfu_style_bw_u, menu->hktxt2[it], &p);
-			g_print_text(dev, p, y, bfu_style_bw, menu->hktxt3[it], &p);
+			g_print_text(dev, p, y, style_bw, menu->hktxt1[it], &p);
+			g_print_text(dev, p, y, style_bw_u, menu->hktxt2[it], &p);
+			g_print_text(dev, p, y, style_bw, menu->hktxt3[it], &p);
+			if (menu->items[it].free_i & MENU_FONT_LIST) {
+				g_free_style(style_bw);
+			}
 		}
 		if (!*rtext) {
 			set_clip_area(dev, &r);
@@ -506,7 +543,7 @@ static void menu_func(struct window *win, struct links_event *ev, int fwd)
 						struct mainmenu *m2 = w1->data;
 						if (F && !f && ev->x >= m2->xl1 && ev->x < m2->xl2 && ev->y >= m2->yl1 && ev->y < m2->yl2) goto bbb;
 #endif
-						if (ev->y < gf_val(1, G_BFU_FONT_SIZE)) goto del;
+						if (ev->y < LL) goto del;
 						break;
 					}
 					if (w1->handler != menu_func) break;
@@ -570,13 +607,13 @@ go_lr:
 			else if (ev->x == KBD_HOME || (upcase(ev->x) == 'A' && ev->y & KBD_CTRL)) menu->selected = -1, scroll_menu(menu, 1);
 			else if (ev->x == KBD_END || (upcase(ev->x) == 'E' && ev->y & KBD_CTRL)) menu->selected = menu->ni, scroll_menu(menu, -1);
 			else if (ev->x == KBD_PAGE_UP || (upcase(ev->x) == 'B' && ev->y & KBD_CTRL)) {
-				if ((menu->selected -= menu->yw / gf_val(1, G_BFU_FONT_SIZE) - 3) < -1) menu->selected = -1;
-				if ((menu->view -= menu->yw / gf_val(1, G_BFU_FONT_SIZE) - 2) < 0) menu->view = 0;
+				if ((menu->selected -= menu->yw / LL - 3) < -1) menu->selected = -1;
+				if ((menu->view -= menu->yw / LL - 2) < 0) menu->view = 0;
 				scroll_menu(menu, -1);
 			}
 			else if (ev->x == KBD_PAGE_DOWN || (upcase(ev->x) == 'F' && ev->y & KBD_CTRL)) {
-				if ((menu->selected += menu->yw / gf_val(1, G_BFU_FONT_SIZE) - 3) > menu->ni) menu->selected = menu->ni;
-				if ((menu->view += menu->yw / gf_val(1, G_BFU_FONT_SIZE) - 2) >= menu->ni - menu->yw + 2) menu->view = menu->ni - menu->yw + 2;
+				if ((menu->selected += menu->yw / LL - 3) > menu->ni) menu->selected = menu->ni;
+				if ((menu->view += menu->yw / LL - 2) >= menu->ni - menu->yw + 2) menu->view = menu->ni - menu->yw + 2;
 				scroll_menu(menu, 1);
 			}
 			else if (ev->x > ' ') {
@@ -614,11 +651,12 @@ go_lr:
 			if (menu->items->free_i) {
 				int i;
 				for (i = 0; i < menu->ni; i++) {
-					if (menu->items[i].free_i & 2) mem_free(menu->items[i].text);
-					if (menu->items[i].free_i & 4) mem_free(menu->items[i].rtext);
-					if (menu->items[i].free_i & 8) mem_free(menu->items[i].hotkey);
+					if (menu->items[i].free_i & MENU_FREE_TEXT) mem_free(menu->items[i].text);
+					if (menu->items[i].free_i & MENU_FREE_RTEXT) mem_free(menu->items[i].rtext);
+					if (menu->items[i].free_i & MENU_FREE_HOTKEY) mem_free(menu->items[i].hotkey);
 				}
-				mem_free(menu->items);
+				if (menu->items->free_i & MENU_FREE_ITEMS)
+					mem_free(menu->items);
 			}
 			if (menu->free_function)
 				register_bottom_half(menu->free_function, menu->free_data);
@@ -745,7 +783,7 @@ static void mainmenu_func(struct window *win, struct links_event *ev, int fwd)
 			draw_to_window(win, display_mainmenu, menu);
 			break;
 		case EV_MOUSE:
-			in_menu = ev->x >= 0 && ev->x < win->term->x && ev->y >= 0 && ev->y < gf_val(1, G_BFU_FONT_SIZE);
+			in_menu = ev->x >= 0 && ev->x < win->term->x && ev->y >= 0 && ev->y < LL;
 			if ((ev->b & BM_ACT) == B_MOVE) break;
 			if ((ev->b & BM_BUTT) == B_FOURTH) {
 				if ((ev->b & BM_ACT) == B_DOWN) goto go_left;
@@ -832,7 +870,7 @@ void add_to_menu(struct menu_item **mi, unsigned char *text, unsigned char *rtex
 	int n;
 	if (pos != -1) {
 		n = pos;
-		if ((*mi)[n].text) internal("invalid menu position %d", n);
+		if ((*mi)[n].text) internal_error("invalid menu position %d", n);
 	} else {
 		for (n = 0; (*mi)[n].text; n++) if (n == MAXINT) overalloc();
 	}
@@ -924,7 +962,7 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
 			}
 			break;
 		default:
-			internal("display_dlg_item: unknown item: %d", di->item->type);
+			internal_error("display_dlg_item: unknown item: %d", di->item->type);
 #ifdef G
 	} else {
 		struct rect rr;
@@ -1034,7 +1072,7 @@ void display_dlg_item(struct dialog_data *dlg, struct dialog_item_data *di, int
 				if (sel) set_window_ptr(dlg->win, di->x, di->y + G_BFU_FONT_SIZE);
 				break;
 			default:
-				internal("display_dlg_item: unknown item: %d", di->item->type);
+				internal_error("display_dlg_item: unknown item: %d", di->item->type);
 		}
 		if (!dlg->s) set_clip_area(dev, &rr);
 #endif
@@ -1238,7 +1276,7 @@ static void do_tab_compl(struct terminal *term, struct list_head *history, struc
 			items[n].rtext = cast_uchar "";
 			items[n].data = hi;
 			items[n].in_m = 0;
-			items[n].free_i = 3;
+			items[n].free_i = MENU_FREE_ITEMS | MENU_FREE_TEXT;
 			if (n == MAXINT) overalloc();
 			n++;
 		} else {
@@ -1569,13 +1607,13 @@ go_prev:
 /* gid and gnum are 100 times greater than boundaries (e.g. if gid==1 boundary is 0.01) */
 int check_float(struct dialog_data *dlg, struct dialog_item_data *di)
 {
-	unsigned char *end;
-	double d = strtod(cast_const_char di->cdata, (char **)(void *)&end);
+	char *end;
+	double d = strtod(cast_const_char di->cdata, &end);
 	if (!*di->cdata || *end || di->cdata[strspn(cast_const_char di->cdata, "0123456789.")] || *di->cdata == (unsigned char)'.') {
 		msg_box(dlg->win->term, NULL, TEXT_(T_BAD_NUMBER), AL_CENTER, TEXT_(T_NUMBER_EXPECTED), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		return 1;
 	}
-	if (d < 0 || d > di->item->gnum || 100*d < di->item->gid || 100*d > di->item->gnum) {
+	if (d < 0 || d > di->item->gnum || 100 * d < di->item->gid || 100 * d > di->item->gnum) {
 		msg_box(dlg->win->term, NULL, TEXT_(T_BAD_NUMBER), AL_CENTER, TEXT_(T_NUMBER_OUT_OF_RANGE), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		return 1;
 	}
@@ -1584,8 +1622,8 @@ int check_float(struct dialog_data *dlg, struct dialog_item_data *di)
 
 int check_number(struct dialog_data *dlg, struct dialog_item_data *di)
 {
-	unsigned char *end;
-	long l = strtol(cast_const_char di->cdata, (char **)(void *)&end, 10);
+	char *end;
+	long l = strtol(cast_const_char di->cdata, &end, 10);
 	if (!*di->cdata || *end) {
 		msg_box(dlg->win->term, NULL, TEXT_(T_BAD_NUMBER), AL_CENTER, TEXT_(T_NUMBER_EXPECTED), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		return 1;
@@ -1599,8 +1637,8 @@ int check_number(struct dialog_data *dlg, struct dialog_item_data *di)
 
 int check_hex_number(struct dialog_data *dlg, struct dialog_item_data *di)
 {
-	unsigned char *end;
-	long l = strtol(cast_const_char di->cdata, (char **)(void *)&end, 16);
+	char *end;
+	long l = strtol(cast_const_char di->cdata, &end, 16);
 	if (!*di->cdata || *end) {
 		msg_box(dlg->win->term, NULL, TEXT_(T_BAD_NUMBER), AL_CENTER, TEXT_(T_NUMBER_EXPECTED), MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 		return 1;
@@ -1655,7 +1693,7 @@ static int check_local_ip_address_internal(struct dialog_data *dlg, struct dialo
 			&& errno != ENOMEM
 #endif
 			) {
-				unsigned char *er = stracpy(cast_uchar strerror(errno));
+				unsigned char *er = strerror_alloc(errno, dlg->win->term);
 				unsigned char *ad = stracpy(p);
 				msg_box(dlg->win->term, getml(er, ad, NULL), TEXT_(T_BAD_IP_ADDRESS), AL_CENTER, TEXT_(T_UNABLE_TO_USE_LOCAL_IP_ADDRESS), cast_uchar " ", ad, cast_uchar ": ", er, MSG_BOX_END, NULL, 1, TEXT_(T_CANCEL), msg_box_null, B_ENTER | B_ESC);
 				return 1;
@@ -1996,7 +2034,7 @@ void dlg_format_buttons(struct dialog_data *dlg, struct terminal *term, struct d
 				p += (butt[i].l = txtlen(dlg->win->term, get_text_translation(butt[i].item->text, dlg->win->term)) + gf_val(4, txtlen(dlg->win->term, cast_uchar(G_DIALOG_BUTTON_L G_DIALOG_BUTTON_R)))) + gf_val(2, G_DIALOG_BUTTON_SPACE);
 			}
 		}
-		*y += gf_val(2, G_BFU_FONT_SIZE * 2);
+		*y += 2 * LL;
 		i1 = i2;
 	}
 }
@@ -2037,12 +2075,15 @@ void dlg_format_field(struct dialog_data *dlg, struct terminal *term, struct dia
 {
 	if (dlg->win->term->spec->braille) w = dlg->win->term->x;
 	if (term) {
+		int l = gf_val(item->item->dlen, item->item->dlen * G_DIALOG_FIELD_WIDTH);
 		item->x = x;
 		item->y = *y;
 		item->l = w;
-		if (rw && item->l > *rw) if ((*rw = item->l) > w) *rw = w;
+		if (item->l > l)
+			item->l = l;
+		if (rw && item->l > *rw) *rw = item->l;
 	}
-	(*y) += gf_val(1, G_BFU_FONT_SIZE);
+	(*y) += LL;
 }
 
 void dlg_format_text_and_field(struct dialog_data *dlg, struct terminal *term, unsigned char *text, struct dialog_item_data *item, int x, int *y, int w, int *rw, unsigned char co, int align)
@@ -2120,11 +2161,11 @@ void dlg_format_group(struct dialog_data *dlg, struct terminal *term, unsigned c
 		if (dlg->win->term->spec->braille) {
 			if (!f) {
 				nx = 0;
-				(*y) += gf_val(1, G_BFU_FONT_SIZE * 1);
+				(*y) += LL;
 			} else f = 0;
 		} else if (nx && nx + wx > w) {
 			nx = 0;
-			(*y) += gf_val(2, G_BFU_FONT_SIZE * 2);
+			(*y) += 2 * LL;
 		}
 		if (term) {
 			if (!F) print_text(term, x + nx + 4 * (item->item->type == D_CHECKBOX), *y, ttxtlen(term, get_text_translation(texts[0], dlg->win->term)), get_text_translation(texts[0], dlg->win->term), COLOR_DIALOG_TEXT);
@@ -2145,11 +2186,9 @@ void dlg_format_group(struct dialog_data *dlg, struct terminal *term, unsigned c
 		texts++;
 		item++;
 	}
-	(*y) += gf_val(1, G_BFU_FONT_SIZE);
+	(*y) += LL;
 }
 
-#define LL gf_val(1, G_BFU_FONT_SIZE)
-
 void checkbox_list_fn(struct dialog_data *dlg)
 {
 	struct terminal *term = dlg->win->term;
@@ -2344,7 +2383,7 @@ void add_to_history(struct terminal *term, struct history *h, unsigned char *t)
 	while (h->n > MAX_HISTORY_ITEMS) {
 		struct history_item *hd;
 		if (list_empty(h->items)) {
-			internal("history is empty");
+			internal_error("history is empty");
 			h->n = 0;
 			return;
 		}
@@ -2383,7 +2422,7 @@ static void input_field_fn(struct dialog_data *dlg)
 	int max = 0, min = 0;
 	int w, rw;
 	int y = gf_val(-1, -G_BFU_FONT_SIZE);
-	if (dlg->win->term->spec->braille) y += gf_val(1, G_BFU_FONT_SIZE);
+	if (dlg->win->term->spec->braille) y += LL;
 	max_text_width(term, dlg->dlg->udata, &max, AL_LEFT);
 	min_text_width(term, dlg->dlg->udata, &min, AL_LEFT);
 	max_buttons_width(term, dlg->items + 1, dlg->n - 1, &max);
@@ -2402,7 +2441,7 @@ static void input_field_fn(struct dialog_data *dlg)
 	center_dlg(dlg);
 	draw_dlg(dlg);
 	y = dlg->y + DIALOG_TB;
-	if (dlg->win->term->spec->braille) y += gf_val(1, G_BFU_FONT_SIZE);
+	if (dlg->win->term->spec->braille) y += LL;
 	dlg_format_text_and_field(dlg, term, dlg->dlg->udata, dlg->items, dlg->x + DIALOG_LB, &y, w, NULL, COLOR_DIALOG_TEXT, AL_LEFT);
 	y += LL;
 	dlg_format_buttons(dlg, term, dlg->items + 1, dlg->n - 1, dlg->x + DIALOG_LB, &y, w, NULL, AL_CENTER);
