@@ -84,7 +84,7 @@ static_const float fancy_constants[64]={
  * a segfault. 65536 pixels wide bitmaps are not normal and will anyway
  * exhaust the memory.
  */
-static inline int compute_width(int ix, int iy, int required_height)
+int compute_width(int ix, int iy, int required_height)
 {
 	int width;
 	unsigned long reg;
