@@ -53,7 +53,11 @@
 /* We assume here int holds at least 32 bits */
 static int *red_table = DUMMY, *green_table = DUMMY, *blue_table = DUMMY;
 static int table_16 = 1;
-static unsigned short *real_colors_table = NULL;
+static unsigned short *
+#if defined(__GNUC__) && __GNUC__ == 3
+volatile
+#endif
+real_colors_table = NULL;
 
 /* If we want to represent some 16-bit from-screen-light, it would require certain display input
  * value (0-255 red, 0-255 green, 0-255 blue), possibly not a whole number. [red|green|blue]_table
