@@ -178,7 +178,7 @@ static void stat_date(unsigned char **p, int *l, struct stat *stp)
 	static unsigned char fmt1[] = "%b %e  %Y";
 	static unsigned char fmt2[] = "%b %e %H:%M";
 	int wr;
-	EINTRLOOPX(current_time, time(NULL), (time_t)-1);
+	current_time = get_absolute_seconds();
 	if (!stp) {
 		wr = 0;
 		goto set_empty;
