@@ -325,6 +325,13 @@ links_ssl *getSSL(void)
 			ssl_load_private_certificates(ctx);
 #endif
 		}
+#ifdef X509_V_FLAG_TRUSTED_FIRST
+#if defined(OPENSSL_VERSION_NUMBER) && OPENSSL_VERSION_NUMBER < 0x10003000
+		X509_VERIFY_PARAM_set_flags(SSL_CTX_get_cert_store(ctx)->param, X509_V_FLAG_TRUSTED_FIRST);
+#else
+		/*X509_VERIFY_PARAM_set_flags(X509_STORE_get0_param(SSL_CTX_get_cert_store(ctx)), X509_V_FLAG_TRUSTED_FIRST);*/
+#endif
+#endif
 		SSL_CTX_set_default_passwd_cb(ctx, ssl_password_callback);
 	}
 	ssl = mem_alloc_mayfail(sizeof(links_ssl));
