@@ -129,13 +129,24 @@ void kbd_ctrl_c(void)
 unsigned char init_seq[] = "\033)0\0337";
 unsigned char init_seq_x_mouse[] = "\033[?1000h\033[?1002h\033[?1005l\033[?1015l\033[?1006h\033[?2004h";
 unsigned char init_seq_tw_mouse[] = "\033[?9h";
+unsigned char init_seq_xterm[] = "\033[?1049h";
 unsigned char term_seq[] = "\033[2J\0338\r \b";
 unsigned char term_seq_x_mouse[] = "\033[?1000l\r       \r\033[?1002l\r       \r\033[?1006l\r       \r\033[?2004l\r       \r";
 unsigned char term_seq_tw_mouse[] = "\033[?9l";
+unsigned char term_seq_xterm[] = "\033[?1049l";
+
+static int use_xterm_save_restore(void)
+{
+	unsigned char *e;
+	return (e = cast_uchar getenv("TERM")) && !casecmp(e, cast_uchar "xterm", 5);
+}
 
 static void send_init_sequence(int h, int flags)
 {
+	save_terminal();
 	want_draw();
+	if (use_xterm_save_restore())
+		hard_write(h, init_seq_xterm, (int)strlen(cast_const_char init_seq_xterm));
 	hard_write(h, init_seq, (int)strlen(cast_const_char init_seq));
 #if !defined(DOS) && !defined(HAIKU)
 	if (flags & USE_TWIN_MOUSE) {
@@ -150,7 +161,6 @@ static void send_init_sequence(int h, int flags)
 static void send_term_sequence(int h, int flags)
 {
 	want_draw();
-	hard_write(h, term_seq, (int)strlen(cast_const_char term_seq));
 #if !defined(DOS) && !defined(HAIKU)
 	if (flags & USE_TWIN_MOUSE) {
 		hard_write(h, term_seq_tw_mouse, (int)strlen(cast_const_char term_seq_tw_mouse));
@@ -158,7 +168,11 @@ static void send_term_sequence(int h, int flags)
 		hard_write(h, term_seq_x_mouse, (int)strlen(cast_const_char term_seq_x_mouse));
 	}
 #endif
+	hard_write(h, term_seq, (int)strlen(cast_const_char term_seq));
+	if (use_xterm_save_restore())
+		hard_write(h, term_seq_xterm, (int)strlen(cast_const_char term_seq_xterm));
 	done_draw();
+	restore_terminal();
 }
 
 static void resize_terminal(int x, int y)
@@ -479,9 +493,7 @@ void fatal_tty_exit(void)
 #ifdef G
 	if (drv && drv->emergency_shutdown) drv->emergency_shutdown();
 #endif
-#ifdef DOS
-	dos_restore_screen();
-#endif
+	restore_terminal();
 }
 
 static void refresh_terminal_size(void)
